
#include "NalParser.h"

int getNalType(const unsigned char * data) {
  unsigned char c = data[2];
  if (c == 0x00) {
    c = data[4];
  } else {
    c = data[3];
  }
  return c&0x1f;
}
void 	  dump10byte(unsigned char *naldata)
{
	printf("%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x \n",
		naldata[0],naldata[1],naldata[2],naldata[3],naldata[4],
		naldata[5],naldata[6],naldata[7],naldata[8],naldata[9]);
}

int
main(int argc, char * argv[]) {
  int bUsageCorrect = 0;
  NalParser * parser1 = NULL;
  NalParser * parser2 = NULL;
  FILE      * fout = NULL;
  int       start1 = -1;
  int       end1   = -1;
  int       start2 = -1;
  int       end2   = -1;
  char      name1[512];
  char      name2[512];

  if (argc == 4) {
	if (argv[1][0] == '-' && argv[1][1] == 'o') {
	  fout = fopen(argv[1]+2, "wb");
	  if (fout != NULL) {
		{
		  char *p = argv[2];
		  while (*p) {
			if (*p == ':') *p = ' ';
			p++;
		  }
		}
		{
		  char *p = argv[3];
		  while (*p) {
			if (*p == ':') *p = ' ';
			p++;
		  }
		}
		if ( (sscanf(argv[2], "%s %d %d", name1, &start1, &end1) == 3) &&
			 (sscanf(argv[3], "%s %d %d", name2, &start2, &end2) == 3) )
		  bUsageCorrect = 1;
	  }
	}
  }
  if (bUsageCorrect == 0) {
	printf("Usage: ElementaryStreamMerger -o{OutputFile} file1:startframe:endframe file2:startframe:endframe\n");
	return 1;
  }

  parser1 = new NalParser(name1);
  parser2 = new NalParser(name2);
  unsigned char * naldata = new unsigned char [1000000];

  uint32 nNalSize;

  int fno = 0;
  int done = 0;
  while (!done) {
	int bReadMore = parser1->getNextNal(naldata, 1000000, &nNalSize);
	if (nNalSize < 0) {
	  fprintf(stderr, "immature death in elementarystreammerger\n");
	}
	// copy sps, pps...
	int naltype = getNalType(naldata);
	if (naltype == 7 || naltype == 8) {
	  //	    printf("--- nalsize = %d\n", nNalSize);
	  fwrite(naldata, nNalSize, 1, fout);
	} else {
	  if (fno >= start1 && fno <= end1) {
	    printf("fno = %d (nalsize = %d)\t", fno, nNalSize);
		dump10byte(naldata);
	    fwrite(naldata, nNalSize, 1, fout);
	  }
	  fno++;
	}
	if (fno > end1)
	  done = 1;
  }
  
  fno = 0;
  done = 0;

  printf("start2 = %d, end2 = %d\n", start2, end2);
  while (!done) {
	int bReadMore = parser2->getNextNal(naldata, 1000000, &nNalSize);
	if (nNalSize < 0) {
	  fprintf(stderr, "immature death in elementarystreammerger\n");
	}
	// do not copy sps, pps...
	int naltype = getNalType(naldata);

	if (!(naltype == 7 || naltype == 8)) {
		if (fno >= start2 && fno <= end2) {
		  printf("fno = %d (nNalSize=%d)\t", fno, nNalSize);
		  dump10byte(naldata);
		  fwrite(naldata, nNalSize, 1, fout);
		}
		fno++;
	}
	if (fno > end2)
	  done = 1;
  }
  
  delete [] naldata;
  return 0;
}
	
