#ifndef PEEKABLE_STREAM_H
#define PEEKABLE_STREAM_H

#include <stdio.h>
#include <stdlib.h>
#include "myType.h"

#define CACHE_BUF_SIZE (16*1024)

class PeekableStream {
protected:
  uint8    *buf_;
  uint32   curpos_;       // pos in buffer
  uint32   nleft_;        // unread bytes left in buffer

  uint32   peekval_;      // value for peeking
  uint32   nReadAhead_;   // number of bytes read ahead
  int8     eof_;

  FILE     *f_;

  void     RefillBuffer() {
	int nRead = fread(buf_, 1, CACHE_BUF_SIZE, f_);
	nleft_ = nRead;
	if (nRead != CACHE_BUF_SIZE)
	  eof_ = 1;
	curpos_ = 0;
  }

public:
  PeekableStream(const char * fname) {
	f_ = fopen(fname, "rb");
	if (f_ == NULL) {
	  fprintf(stderr, "Cannot open file %s. exiting...\n", fname);
	  exit(1);
	}
	buf_ = new uint8 [CACHE_BUF_SIZE];
	if (buf_ == NULL) {
	  fprintf(stderr, "malloc failed.  exiting...\n");
	  exit(1);
	}
	RefillBuffer();
	peekval_ = (buf_[0] << 24) | (buf_[1] << 16) | (buf_[2] << 8) | buf_[3];
	curpos_ = 4;
	nReadAhead_ = 4;
	nleft_ -= 4;
	eof_ = 0;
  }
  ~PeekableStream() {

  }
  int8     bEof()           const { return eof_ && (nleft_ == 0) && (nReadAhead_ == 0); }
  uint32   PeekThreeBytes() const { return peekval_>>8; }
  uint32   PeekFourBytes()  const { return peekval_; }

  /*
   * better check bEof better calling this...
   */
  uint8    getByte() {
	uint8  rv = 0;
	if (bEof() == 0) {
	  rv = (peekval_>>24)&0x00ff;
	  // advance byte...
	  if (nleft_ == 0 && eof_ == 0) {
		RefillBuffer();
	  }
	  if (nleft_ > 0) {
		peekval_ = (peekval_<<8) | buf_[curpos_++];
		nleft_--;
	  } else if (nReadAhead_ > 0) {
		// falling here means only bytes in "peekval" left...
		rv = peekval_>>(8*(nReadAhead_-1));
		nReadAhead_--;
	  } 
	  return rv;
	}
  }
};

#endif // PEEKABLE_STREAM_H

